rsocket-branch: Update RDMA selector provider and API rsocket-branch
authorchegar
Wed, 30 Jan 2019 14:24:11 +0000
branchrsocket-branch
changeset 57123 919516f93dcf
parent 57121 ff7b5750a610
child 57129 1e1db86ea836
rsocket-branch: Update RDMA selector provider and API
src/jdk.net/linux/classes/jdk/internal/net/rdma/RdmaPollSelectorProvider.java
src/jdk.net/share/classes/jdk/net/RdmaSockets.java
test/jdk/jdk/net/RdmaSockets/rsocket/RdmaSelectorProvider.java
--- a/src/jdk.net/linux/classes/jdk/internal/net/rdma/RdmaPollSelectorProvider.java	Tue Jan 29 17:27:13 2019 +0000
+++ b/src/jdk.net/linux/classes/jdk/internal/net/rdma/RdmaPollSelectorProvider.java	Wed Jan 30 14:24:11 2019 +0000
@@ -25,35 +25,30 @@
 
 package jdk.internal.net.rdma;
 
+import sun.nio.ch.Net;
+
 import java.io.IOException;
 import java.net.ProtocolFamily;
+import java.net.StandardProtocolFamily;
 import java.nio.channels.DatagramChannel;
 import java.nio.channels.Pipe;
 import java.nio.channels.ServerSocketChannel;
 import java.nio.channels.SocketChannel;
 import java.nio.channels.spi.AbstractSelector;
 import java.nio.channels.spi.SelectorProvider;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
+
+import static java.net.StandardProtocolFamily.INET;
+import static java.net.StandardProtocolFamily.INET6;
 
 public class RdmaPollSelectorProvider
     extends SelectorProvider
 {
     private static final Object lock = new Object();
-    private static RdmaPollSelectorProvider provider;
+    private static final RdmaPollSelectorProvider provider =
+            new RdmaPollSelectorProvider();
 
     public static RdmaPollSelectorProvider provider() {
-        synchronized (lock) {
-            if (provider != null)
-                return provider;
-            return AccessController.doPrivileged(
-                new PrivilegedAction<>() {
-                    public RdmaPollSelectorProvider run() {
-                            provider = new RdmaPollSelectorProvider();
-                            return provider;
-                        }
-                    });
-        }
+        return provider;
     }
 
     @Override
@@ -62,8 +57,11 @@
     }
 
     @Override
-    public SocketChannel openSocketChannel() {
-        throw new UnsupportedOperationException();
+    public SocketChannel openSocketChannel() throws IOException {
+        if (Net.isIPv6Available()) {
+            return openSocketChannel(INET6);
+        }
+        return openSocketChannel(INET);
     }
 
     public SocketChannel openSocketChannel(ProtocolFamily family)
@@ -73,8 +71,11 @@
     }
 
     @Override
-    public ServerSocketChannel openServerSocketChannel() {
-        throw new UnsupportedOperationException();
+    public ServerSocketChannel openServerSocketChannel() throws IOException {
+        if (Net.isIPv6Available()) {
+            return openServerSocketChannel(INET6);
+        }
+        return openServerSocketChannel(INET);
     }
 
     public ServerSocketChannel openServerSocketChannel(ProtocolFamily family)
--- /dev/null	Thu Jan 01 00:00:00 1970 +0000
+++ b/test/jdk/jdk/net/RdmaSockets/rsocket/RdmaSelectorProvider.java	Wed Jan 30 14:24:11 2019 +0000
@@ -0,0 +1,109 @@
+/*
+ * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.
+ *
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
+ */
+
+/*
+ * @test
+ * @bug 8195160
+ * @summary Checks the RDMA selector provider API methods
+ * @requires (os.family == "linux")
+ * @library /test/lib
+ * @build RsocketTest
+ * @run testng/othervm RdmaSelectorProvider
+ */
+
+import java.io.IOException;
+import java.net.ProtocolFamily;
+import java.nio.channels.Selector;
+import java.nio.channels.ServerSocketChannel;
+import java.nio.channels.SocketChannel;
+import org.testng.SkipException;
+import org.testng.annotations.BeforeTest;
+import org.testng.annotations.DataProvider;
+import org.testng.annotations.Test;
+import static java.net.StandardProtocolFamily.INET;
+import static java.net.StandardProtocolFamily.INET6;
+import static jdk.net.RdmaSockets.*;
+import static org.testng.Assert.assertNotEquals;
+import static org.testng.Assert.assertThrows;
+
+public class RdmaSelectorProvider {
+
+    @BeforeTest
+    public void setup() throws Exception {
+        if (!RsocketTest.isRsocketAvailable())
+            throw new SkipException("rsocket is not available");
+    }
+
+    @DataProvider(name = "families")
+    public Object[][] families() {
+        return new Object[][] { { INET }, { INET6 } };
+    }
+
+    static final Class<UnsupportedOperationException> UOE = UnsupportedOperationException.class;
+
+    @Test(dataProvider = "families")
+    public void testSocketChannel(ProtocolFamily family)
+        throws IOException
+    {
+        try (SocketChannel sc = openSocketChannel(family)) {
+            assertThrows(UOE, () -> sc.provider().openDatagramChannel());
+            assertThrows(UOE, () -> sc.provider().openDatagramChannel(INET));
+            assertThrows(UOE, () -> sc.provider().openDatagramChannel(INET6));
+            assertThrows(UOE, () -> sc.provider().openPipe());
+
+            assertNotEquals(sc.provider().openSelector(), null);
+            assertNotEquals(sc.provider().openSocketChannel(), null);
+            assertNotEquals(sc.provider().openServerSocketChannel(), null);
+        }
+    }
+
+    @Test(dataProvider = "families")
+    public void testServerSocketChannel(ProtocolFamily family)
+        throws IOException
+    {
+        try (ServerSocketChannel ssc = openServerSocketChannel(family)) {
+            assertThrows(UOE, () -> ssc.provider().openDatagramChannel());
+            assertThrows(UOE, () -> ssc.provider().openDatagramChannel(INET));
+            assertThrows(UOE, () -> ssc.provider().openDatagramChannel(INET6));
+            assertThrows(UOE, () -> ssc.provider().openPipe());
+
+            assertNotEquals(ssc.provider().openSelector(), null);
+            assertNotEquals(ssc.provider().openSocketChannel(), null);
+            assertNotEquals(ssc.provider().openServerSocketChannel(), null);
+        }
+    }
+
+    @Test
+    public void testSelector() throws IOException {
+        try (Selector selector = openSelector()) {
+            assertThrows(UOE, () -> selector.provider().openDatagramChannel());
+            assertThrows(UOE, () -> selector.provider().openDatagramChannel(INET));
+            assertThrows(UOE, () -> selector.provider().openDatagramChannel(INET6));
+            assertThrows(UOE, () -> selector.provider().openPipe());
+
+            assertNotEquals(selector.provider().openSelector(), null);
+            assertNotEquals(selector.provider().openSocketChannel(), null);
+            assertNotEquals(selector.provider().openServerSocketChannel(), null);
+        }
+    }
+}
--- a/src/jdk.net/share/classes/jdk/net/RdmaSockets.java	Tue Jan 29 17:27:13 2019 +0000
+++ b/src/jdk.net/share/classes/jdk/net/RdmaSockets.java	Wed Jan 30 14:24:11 2019 +0000
@@ -28,35 +28,45 @@
 import java.net.ProtocolFamily;
 import java.net.ServerSocket;
 import java.net.Socket;
+import java.net.StandardProtocolFamily;
 import java.nio.channels.Selector;
 import java.nio.channels.ServerSocketChannel;
 import java.nio.channels.SocketChannel;
+import java.nio.channels.spi.SelectorProvider;
 import java.io.IOException;
 import java.util.Objects;
 import jdk.internal.net.rdma.RdmaPollSelectorProvider;
 import jdk.internal.net.rdma.RdmaSocketProvider;
 
 /**
- * Factory methods for creating RDMA-based TCP sockets and channels.
+ * Factory methods for creating RDMA-based TCP sockets and socket channels.
  *
- * <p>The {@link #openSocket(ProtocolFamily family) openSocket} and {@link
- * #openServerSocket(ProtocolFamily family) openServerSocket} methods
- * create RDMA-based TCP sockets.
+ * <p>The {@link #openSocket(ProtocolFamily) openSocket} and {@link
+ * #openServerSocket(ProtocolFamily) openServerSocket} methods create RDMA-based
+ * TCP sockets.
  *
  * <p>The {@link #openSelector() openSelector}, {@link
  * #openSocketChannel(ProtocolFamily family) openSocketChannel}, and {@link
  * #openServerSocketChannel(ProtocolFamily family) openServerSocketChannel}
  * methods create selectors and selectable channels for use with RDMA sockets.
  * These objects are created by a {@link java.nio.channels.spi.SelectorProvider
  * SelectorProvider} that is not the default {@code SelectorProvider}.
  * Consequently, selectable channels to RDMA sockets may not be multiplexed
  * with selectable channels created by the default selector provider. Its
  * selector provider does not support datagram channels and pipes.
  * The {@link java.nio.channels.spi.SelectorProvider#openDatagramChannel
  * openDatagramChannel} and
  * {@link java.nio.channels.spi.SelectorProvider#openPipe openPipe} methods
  * throw {@link java.lang.UnsupportedOperationException
  * UnsupportedOperationException}.
  *
  * @since 13
  */
@@ -28,35 +28,45 @@
  *
  * <p>The {@link #openSelector() openSelector}, {@link
- * #openSocketChannel(ProtocolFamily family) openSocketChannel}, and {@link
- * #openServerSocketChannel(ProtocolFamily family) openServerSocketChannel}
- * methods create selectors and selectable channels for use with RDMA sockets.
- * These objects are created by a {@link java.nio.channels.spi.SelectorProvider
- * SelectorProvider} that is not the default {@code SelectorProvider}.
- * Consequently, selectable channels to RDMA sockets may not be multiplexed
- * with selectable channels created by the default selector provider. Its
- * selector provider does not support datagram channels and pipes.
- * The {@link java.nio.channels.spi.SelectorProvider#openDatagramChannel
- * openDatagramChannel} and
- * {@link java.nio.channels.spi.SelectorProvider#openPipe openPipe} methods
- * throw {@link java.lang.UnsupportedOperationException
- * UnsupportedOperationException}.
+ * #openSocketChannel(ProtocolFamily) openSocketChannel}, and {@link
+ * #openServerSocketChannel(ProtocolFamily) openServerSocketChannel} methods
+ * create selectors and selectable channels for use with RDMA sockets. These
+ * selectors and channels are created by the RDMA selector provider, which is
+ * not the {@linkplain SelectorProvider#provider() default} system-wide selector
+ * provider. Consequently, selectable channels to RDMA sockets may not be
+ * multiplexed with selectable channels created by the default system-wide
+ * selector provider. The RDMA selector provider does not support datagram
+ * channels or pipes. Its {@link SelectorProvider#openDatagramChannel
+ * openDatagramChannel} and {@link SelectorProvider#openPipe openPipe} methods
+ * throw {@link UnsupportedOperationException UnsupportedOperationException}.
+ *
+ * @implNote The RDMA selector provider supports socket channels of type
+ * {@link StandardProtocolFamily#INET INET} and {@link
+ * StandardProtocolFamily#INET6 INET6}. Its {@link
+ * SelectorProvider#openSocketChannel() openSocketChannel} and {@link
+ * SelectorProvider#openServerSocketChannel() openServerSocketChannel}
+ * methods create selectable channels with a family of {@link
+ * StandardProtocolFamily#INET6 INET6}, if the underlying platform supports
+ * IPv6. Otherwise, it creates selectable channels with a family of {@link
+ * StandardProtocolFamily#INET INET}.
  *
  * @since 13